#!/usr/bin/env python
# coding: utf-8

import numpy as np 
import astropy.cosmology 
import astropy.units as u
import pandas as pd
from astropy.cosmology import FlatLambdaCDM , Planck18


# In[ ]:

class nc_all_sky_divide():
    def __init__(self,file,div_ra,div_dec,H,omegamc,dcondc):
        self.ra = np.array(file.ra)
        self.dec = np.array(file.dec)
        self.z = np.array(file.redshift)
        self.h0c = H 
        self.omegamc = omegamc 
        self.dcondc = dcondc 
        self.div_ra = div_ra
        self.div_dec = div_dec
        self.min_z = min(file.redshift)
        
    def no_com_dec(self):
        dec = self.dec
        z = self.z
        min_z = self.min_z
        div_dec = self.div_dec
        dec_index_parallel_field = []

        alpha = abs(int(max(dec)+1)-int(min(dec)-1))/div_dec
        
            
        cosmo = FlatLambdaCDM(H0 = self.h0c, Om0 = self.omegamc)
        link_len = cosmo.arcsec_per_kpc_proper(min_z)/3600.*self.dcondc*u.kpc/u.arcsec #[degree]
        print("linkling_angular is",link_len,"[degree]")
        print("The length of the common area in the Dec direction is "+str(2*link_len) +"[degree]") 
        beta = min(dec)+((alpha-link_len)*div_dec)
        
        N_good = abs(int(max(dec)+1)-int(min(dec)-1))/(2*link_len)
        if (N_good < self.div_dec):
            print("To run the code, you need to divide the areas into numbers smaller than "+str(int(N_good))+"\n Please reset the option.")
            return None
        
        gamma = (max(dec)-beta)/(alpha-link_len)
        if (gamma == int(gamma)):
            gamma = gamma
        else:
            gamma += 1 
        
        div_dec += int(gamma)

        print("Total divided field :", int(div_dec))
        
        if (div_dec > self.div_dec+5):
            print("I recommend setting the number of divided areas smaller.")
        
        
        f_dec = int(min(dec)-1)
        dec_index_parallel_field = []
        for _ in range(div_dec):
        
            if f_dec != int(min(dec)-1):
                if (f_dec) >=0: 
                    idx = np.where((dec>=(f_dec-link_len))&(dec<=f_dec+alpha))[0].astype(int)
                    f_dec += alpha-link_len
                else:
                    if (f_dec+alpha) >=0:
                        idx = np.where((dec>=f_dec-link_len)&(dec<=f_dec+alpha))[0].astype(int)
                        f_dec += alpha-link_len
                    
                    else:
                        idx = np.where((dec>=f_dec-link_len)&(dec<=f_dec+alpha))[0].astype(int)
                        f_dec += alpha-link_len
                        
            else: 
                idx = np.where(dec<=min(dec)+alpha)[0].astype(int)
                f_dec+= alpha-link_len
            dec_index_parallel_field.append(idx)
        return dec_index_parallel_field
    
    
def save_file(file,div_ra,div_dec,H,omegamc,dcondc,dir2):
    if (div_dec <= 0):
        print("You have set the number of divided fields to a negative value. Please set it to a positive value.")
        return None
    print("Caution) If there are any previously saved files, it is recommended to delete them.")
    lst = nc_all_sky_divide(file,div_ra,div_dec,H,omegamc,dcondc).no_com_dec()
    if  lst == None:
        return None
    for i in range(len(lst)): 
        imfo = np.array([np.array(file.ra)[lst[i]],np.array(file.dec)[lst[i]],np.array(file.redshift)[lst[i]],np.array(file.ID)[lst[i]]]).T
        np.savetxt(dir2+"field_"+str(i),imfo)






#!/usr/bin/env python
# coding: utf-8

# In[ ]:

__all__=["FOF.py","Group_Summary.py","region_merging.py","save_field.py",'2mrs_data_example.ipynb','tng300_data_example.ipynb','tng300_000_ksmag19_b4.hdf','2MRS_add_distance.hdf'] 





#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import numpy as np 
import astropy.cosmology 
from functools import partial 
import os
import astropy.units as u
import pandas as pd
import glob
import matplotlib.pyplot as plt
from astropy.cosmology import FlatLambdaCDM , Planck18
import time
from tqdm import trange
from pydl.goddard.astro import gcirc
import astropy.units as u
import ray
import sys
sys.setrecursionlimit(10**6)


# depth 설정 


# In[ ]:


class fofradecV():
    def __init__(self,h0c,omegamc,omegalc,omegakc,c,sdcondc,vcondc): 
        self.h0c = h0c 
        self.omegamc = omegamc
        self.omegalc = omegalc
        self.omegakc = omegakc
        self.c = c 
        self.sdcondc = sdcondc
        self.vcondc = vcondc
    
    
    def solution(self,i1,k,flag,lst,ra,de,v):
        """
        Descendant Node(s)
        """
        rax = ra[i1]
        dex = de[i1]
        vx = v[i1]
        # 전체에 대한 flag != 1 index 
        range1 = np.where(flag != 1)[0]
        cnt = range1.size 
        if (cnt >= 1):
            
            d12 = gcirc(rax/15,dex,ra[range1]/15,de[range1],1) # 상위 집합  range1
            sdcond1 = self.sdcondc[i1] 
            v12 = abs((vx-v[range1])/(1.+vx/self.c))
            v21 = abs((vx-v[range1])/(1+v[range1]/self.c))
            
            v_idx = np.where((v12<=self.vcondc)|(v21<=self.vcondc))[0]             # 상위 집합 range1 -> v_idx
            
           
            d_idx = np.where((d12[v_idx]<=sdcond1)|(d12[v_idx]<=self.sdcondc[range1[v_idx]]))[0] # range1 -> v_idx -> d_idx
            
            grj = v_idx[d_idx]

            num = grj.size                
            if (num == 0):pass

            if (num == 1):  
                lst[k+1] = range1[grj]
                flag[range1[grj]] = 1
                k += 1 
                
            if (num > 1):
                lst[k+1:k+num+1] = range1[grj]
                flag[range1[grj]] = 1
                k+= num
                
            for j in range(0,num):
                """
                Recrusive Function
                """
                alpha = range1[grj[j]]
                lst,flag,k =  self.solution(alpha,k,flag,lst,ra,de,v)
        return lst,flag,k 


# In[ ]:


@ray.remote
def mainloop(file,H0,omegamc,omegalc,omegakc,c,dcondc,vcondc,nmin):
    print("start")
    ti = time.time()

    field = np.loadtxt(file)
    field = field[field[:,2].argsort()]
    ##########################
    #                        #
    #                        #
    #    common paramters    #
    #                        #
    #                        #
    ##########################

#     fname = ["N"] 
#     fra = [-99.]
#     fde = [-99.]
#     fz = [-99]
#     fnum = [-9.] 
    ra = field[:,0]
    dec = field[:,1]
    z = field[:,2]

    ID = field[:,3].astype("str")
    v=z*c
    
    ngal = len(ra)
    cosmo = FlatLambdaCDM(H0 = H0, Om0 = omegamc)
    sdcondc =  cosmo.arcsec_per_kpc_proper(z)*dcondc*u.kpc/u.arcsec
    gname = np.ones(ngal)*(-1)    
    ngroup = 0 
    flag = np.zeros(ngal)
    for i in range(ngal):
        """
        Root Node(s)
        """
        
        lst = np.array([-1 for _ in range(ngal)])
        if (flag[i] != 0): continue  
        k = 0 
        lst[k] = i 
        flag[i] = 1 
        fof = fofradecV(H0,omegamc,omegalc,omegakc,c,sdcondc,vcondc)
        lst,flag,k = fof.solution(i,k,flag,lst,ra,dec,v)

        if (k >= nmin-1):
            l0 = lst[0]
            ind = np.where(lst >= 0)[0].astype(int)
            cnt = ind.size
            str1 = ID[l0]

            str1 = str1.replace(" ","")
            gname[lst[ind]] = str1
#             fname.append(str1) 

#             toto1 = np.where(ra[lst[ind]] < 90)[0]
#             ntoto1 = toto1.size

#             toto2 = np.where(ra[lst[ind]] > 270)[0]
#             ntoto2 = toto2.size 

    
            ngroup += 1 
    tf = time.time()

#     if ngroup >= 1:
#         fname = fname[1:ngroup+1] 
#         fra = fra[1:ngroup+1]
#         fde = fde[1:ngroup+1]
#         fz = fz[1:ngroup+1]
#         fnum = fnum[1:ngroup+1]
    
    # Time check Min/Sec
    print(f'{(tf-ti)//60} mins {((tf-ti)%60)//1} second')                

    idx = np.where(gname != -1)[0]
    arr = np.array([ID[idx],ra[idx],dec[idx],z[idx],gname[idx]]).astype(float).T

    return arr




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import numpy as np
import pandas as pd 


# In[1]:


# Input : Unique_ID, Ra,Dec,Redshift, Group_ID 
def nmin_cut(result,nmin):
    result_copy = result.copy()
    
    uni_group = np.unique(result_copy[:,4])
    for idx,val in enumerate(uni_group):
        find = np.where(result_copy[:,4] == val)[0]
        if find.size < nmin:
            result_copy = np.delete(result_copy,find,axis=0)
    print("Total Number of Galaxies  :", len(result_copy[:,0]))
    print("Total Number of Finding Groups :", len(np.unique(result_copy[:,4])))
    return result_copy 


# In[ ]:


def group_summary(result):
    # fname,fra,fdec,fredshift,fnum
    uni_group = np.unique(result[:,4])
    gsum = np.zeros([5,len(uni_group)]).T
    
    
    for idx,val in enumerate(uni_group):
        find = np.where(result[:,4] == val)[0]
        
        
        f_ra = np.mean(result[:,1][find])
        f_dec = np.mean(result[:,2][find])
        f_redshift = np.mean(result[:,3][find])
        f_ID = val 
        f_num = len(find)
        
        gsum[idx] = np.array([f_ID,f_ra,f_dec,f_redshift,f_num])
    
    relabeling = np.arange(1,len(uni_group))
    
    nmin_col = gsum[:,4]
    
    sorted_indices = np.argsort(nmin_col)[::-1]
    sorted_gsum = gsum[sorted_indices]
    
    relabeling_gname = np.arange(1,len(nmin_col)+1)
    for idx,val in enumerate(relabeling_gname):
        sorted_gsum[:,0][idx] = val
    
    return sorted_gsum




#!/usr/bin/env python
# coding: utf-8

# In[1]:


import numpy as np
from tqdm import trange

# In[1]:
def renew_region_merging(parallel_field1,parallel_field2):
    final_group1 = np.zeros(len(parallel_field1[:,4]))
    final_group2 = np.zeros(len(parallel_field2[:,4]))
    
    parallel_field1_copy = parallel_field1.copy()
    parallel_field2_copy = parallel_field2.copy()
    
    
    grid1 = parallel_field1[:,4].copy()
    grid2 = parallel_field2[:,4].copy()
    
    com_part,paral1_com_part_idx,paral2_com_part_idx = np.intersect1d(parallel_field1[:,0],parallel_field2[:,0],return_indices=True)
    
    for i in range(len(paral1_com_part_idx)):
        if (grid1[paral1_com_part_idx[i]] == grid2[paral2_com_part_idx[i]]):
            continue
        ind1 = np.where(grid1 == grid1[paral1_com_part_idx[i]])[0]
        ind2 = np.where(grid2 == grid2[paral2_com_part_idx[i]])[0]
        if (any(final_group1[ind1] ==0)|any(final_group2[ind2] ==0)):
            tmpid = grid1[paral1_com_part_idx[i]]
        else:
            tmpid = final_group1[paral1_com_part_idx[i]]
        find_revision_group = np.where(grid1 == grid2[paral2_com_part_idx[i]])[0]
        
        grid1[ind1] = tmpid
        grid2[ind2] = tmpid
        grid1[find_revision_group] = tmpid
        final_group1[ind1] = tmpid
        final_group2[ind2] = tmpid 
        final_group1[find_revision_group] = tmpid
        
    parallel_field1_copy[:,4] = grid1
    parallel_field2_copy[:,4] = grid2
    
    
    parallel_field1_copy = np.concatenate((parallel_field1_copy,parallel_field2_copy))
    parallel_field1_copy = np.flip(parallel_field1_copy,axis=0)
    
    tot_idx = np.unique(parallel_field1_copy[:,0],return_index= True )[1]
    
    return parallel_field1_copy[tot_idx]



# In[ ]:

def Total_merging(parallel_result):
    result= renew_region_merging(parallel_result[0],parallel_result[1])
    for i in range(1,len(parallel_result)-1):
        result = renew_region_merging(result,parallel_result[i+1])
    return result

def nmin_unique(fields):
    lst = np.array([])
    for idx,val in enumerate(np.unique(fields[:,4])):
        find = np.where(fields[:,4] == val)[0].astype(int)
        if find.size <= 4 :
            fields = np.delete(fields,find,axis=0)
    return fields 




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from setuptools import setup, find_packages

with open('README.md', encoding='utf-8') as f: # README.md 내용 읽어오기
	  long_description = f.read()

setup(
	name='CNUFOF_utils',
	version='1.0.6', 
	long_description = long_description, 
	long_description_content_type = 'text/markdown', 
	description='Parallalization Friends Of Friends based on observation data.', 
	author='Dongyun Kwak', 
	author_email='98ehddbs@naver.com', 
	url='https://github.com/dongyun-gwag', 
	license='MIT', # 라이센스 등록
	python_requires='>= 3.9.18', #파이썬 버전 등록
	install_requires=[ 'ray', 'astropy','pandas','pydl','tqdm'], # module 필요한 다른 module 등록
	packages_data={'functions':'*.py'} # 업로드할 module이 있는 폴더 입력
)

